package com.br.vini.compose.api

import com.br.fechadura.mobile.api.model.Usuario
import com.br.fechadura.mobile.api.request.LoginRequestBody
import com.br.fechadura.mobile.api.response.GrupoPortaResponseBody
import com.br.fechadura.mobile.api.response.GrupoPortasResponseBody
import com.br.fechadura.mobile.api.response.GrupoUsuarioResponseBody
import com.br.fechadura.mobile.api.response.GrupoUsuariosResponseBody
import com.br.fechadura.mobile.api.response.LoginResponseBody
import com.br.fechadura.mobile.api.response.PortaResponseBody
import com.br.fechadura.mobile.api.response.PortasResponseBody
import com.br.fechadura.mobile.api.response.UsuarioResponseBody
import com.br.fechadura.mobile.api.response.UsuariosResponseBody
import retrofit2.Response
import retrofit2.http.Body
import retrofit2.http.GET
import retrofit2.http.POST
import retrofit2.http.Path
import retrofit2.http.Query

/*
Interface que representa o mapeamento dos endpoints da API para métodos correspondentes.
Aqui informamos quais dados serão enviados e recebidos em cada endpoint.

Com esse mapeamento o Retrofit irá:
 - implementar estes métodos para que possa realizar as chamadas a API;
 - converter os dados de envio e resposta em JSON.
 */
interface ApiEndpoint {
    @POST("/login")
    suspend fun login(@Body requestBody: LoginRequestBody) : Response<LoginResponseBody>

    @GET("/allUsuarios")
    suspend fun usuarios() : Response<UsuariosResponseBody>

    @GET("/usuarios/{id}")
    suspend fun getUsuariosById(@Path("id") id: String) : Response<UsuarioResponseBody>

    // Grupo de Usuários
    @GET("/grupoUsuarios")
    suspend fun getGrupoUsuarios(): Response<GrupoUsuariosResponseBody>

    @GET("/grupoUsuarios/{id}")
    suspend fun getGrupoUsuarioById(@Path("id") id: String) : Response<GrupoUsuarioResponseBody>
    // Portas
    @GET("/portas")
    suspend fun getPortas() : Response<PortasResponseBody>

    @GET("/portas/{id}")
    suspend fun getPortaById(@Path("id") id: String) : Response<PortaResponseBody>

    // Grupo de Portas
    @GET("/grupoPortas")
    suspend fun getGrupoPortas() : Response<GrupoPortasResponseBody>

    @GET("/grupoPortas/{id}")
    suspend fun getGrupoPortaById(@Path("id") id: String) : Response<GrupoPortaResponseBody>
}