package com.br.fechadura.mobile.ui.components.grupoUsuario

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.draw.drawBehind
import androidx.compose.ui.geometry.Offset
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import com.br.fechadura.mobile.R
import com.br.fechadura.mobile.api.model.GrupoUsuario


@Composable
fun TabelaGrupoUsuario(grupoUsuario: GrupoUsuario) {
    Surface(
        color = Color.White
    ) {
        Row(
            Modifier
                .fillMaxWidth()
                .height(80.dp)
                .drawBehind {
                    val borderSize = 1.dp.toPx()
                    drawLine(
                        color = Color.Gray,
                        start = Offset(0f, size.height),
                        end = Offset(size.width, size.height),
                        strokeWidth = borderSize
                    )
                },
            verticalAlignment = Alignment.CenterVertically,
            horizontalArrangement = Arrangement.SpaceBetween
        ) {
            // Conteúdo clicável para visualizar o grupo de usuário
            Row(
                modifier = Modifier
                    .weight(1f)
                    .clickable { /* Ação ao clicar para visualizar o grupo de usuário */ }
                    .padding(15.dp),
                verticalAlignment = Alignment.CenterVertically
            ) {
                Column(modifier = Modifier.padding(start = 15.dp)) {
                    Text(
                        text = grupoUsuario.nome,
                        style = MaterialTheme.typography.titleSmall,
                        fontWeight = FontWeight.Black
                    )
                    Text(
                        text = if (grupoUsuario.ativo) "Ativo" else "Não Ativo",
                        modifier = Modifier.width(80.dp)
                    )
                }
            }

            // Ícone clicável para editar o grupo de usuário com área clicável aumentada
            Row(
                modifier = Modifier
                    .width(60.dp)  // Aumenta a área clicável
                    .height(80.dp)
                    .clickable { /* Ação ao clicar para editar o grupo de usuário */ },  // Torna o Row clicável
                verticalAlignment = Alignment.CenterVertically,
                horizontalArrangement = Arrangement.Center
            ) {
                Icon(
                    painter = painterResource(id = R.drawable.editblack),
                    contentDescription = "editar",
                    modifier = Modifier
                        .size(25.dp)  // Tamanho do ícone permanece o mesmo
                )
            }
        }
    }
}