package com.br.fechadura.mobile.ui.screens.grupoPortas

import androidx.compose.foundation.background

import androidx.compose.foundation.clickable
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentSize
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.LazyRow
import androidx.compose.foundation.lazy.items
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material.Divider
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Switch
import androidx.compose.material3.SwitchDefaults
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.unit.dp
import androidx.navigation.NavController
import com.br.fechadura.mobile.api.model.GrupoPorta
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.ArrowDropDown
import androidx.compose.material.icons.filled.Check
import androidx.compose.material.icons.filled.Close
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.ModalBottomSheet
import androidx.compose.material3.Surface
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.material3.rememberModalBottomSheetState
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.ui.graphics.Color.Companion.Black
import androidx.compose.ui.graphics.Color.Companion.Gray
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.unit.sp
import com.br.fechadura.mobile.sampledata.sampleDataGroupDoors
import com.br.fechadura.mobile.ui.theme.FechaduraColor
import kotlinx.coroutines.launch

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun EditarGrupoPorta(
    navController: NavController,
    id: String,
    grupoPorta: GrupoPorta = sampleDataGroupDoors.first()
) {
    var nome by remember { mutableStateOf(grupoPorta.nome) }
    var descricao by remember { mutableStateOf(grupoPorta.descricao) }
    var ativo by remember { mutableStateOf(grupoPorta.ativo) }
    var portas by remember { mutableStateOf(grupoPorta.portas) }

    var selectedPortas by remember { mutableStateOf(setOf<String>()) }
    var selectedText by remember { mutableStateOf("Selecione as portas") }
    var searchQuery by remember { mutableStateOf("") }

    val sheetState = rememberModalBottomSheetState()
    val scope = rememberCoroutineScope()
    var showBottomSheet by remember { mutableStateOf(false) }

    Scaffold(
        topBar = {
            androidx.compose.material3.TopAppBar(
                title = {
                    Text(nome ?: "Visualizar Grupo Porta")
                },
                navigationIcon = {
                    IconButton(onClick = { navController.popBackStack() }) {
                        Icon(
                            imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                            contentDescription = "Voltar",
                            tint = Color.White
                        )
                    }
                },
                colors = TopAppBarDefaults.topAppBarColors(
                    containerColor = FechaduraColor,
                    actionIconContentColor = Gray,
                    navigationIconContentColor = Gray,
                    scrolledContainerColor = Gray,
                    titleContentColor = Color.White
                )
            )
        },
    ) { innerPadding ->
        Box(
            modifier = Modifier
                .fillMaxSize()
                .background(Color.White)
                .padding(innerPadding)
        ) {
            Column(
                modifier = Modifier
                    .fillMaxSize()
                    .padding(24.dp),
                verticalArrangement = Arrangement.SpaceBetween // Alinhamento vertical entre os elementos
            ) {
                Column(
                    modifier = Modifier.weight(1f) // Permite que o conteúdo principal ocupe a maior parte do espaço
                ) {
                    OutlinedTextField(
                        value = nome,
                        onValueChange = {},
                        label = {
                            Text(
                                "Nome",
                                color = FechaduraColor
                            )
                        },
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(top = 16.dp),
                        enabled = false,
                        textStyle = TextStyle(color = Black),
                        colors = TextFieldDefaults.outlinedTextFieldColors(
                            focusedBorderColor = Gray,
                            unfocusedBorderColor = Black
                        ),
                        singleLine = true
                    )

                    OutlinedTextField(
                        value = descricao,
                        onValueChange = {},
                        label = {
                            Text(
                                "Descrição",
                                color = FechaduraColor
                            )
                        },
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(top = 16.dp),
                        enabled = false,
                        textStyle = TextStyle(color = Black),
                        colors = TextFieldDefaults.outlinedTextFieldColors(
                            focusedBorderColor = Gray,
                            unfocusedBorderColor = Black
                        ),
                        singleLine = true
                    )

                    OutlinedTextField(
                        value = "Ver portas do grupo",
                        onValueChange = {},
                        label = {
                            Text(
                                "Descrição",
                                color = FechaduraColor
                            )
                        },
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(top = 16.dp),
                        enabled = false,
                        singleLine = true,
                        textStyle = TextStyle(color = Black),
                        trailingIcon = {
                            IconButton(onClick = { showBottomSheet = true }) {
                                Icon(
                                    imageVector = Icons.Default.ArrowDropDown,
                                    contentDescription = "Abrir menu",
                                    tint = Gray
                                )
                            }
                        },
                        colors = TextFieldDefaults.outlinedTextFieldColors(
                            focusedBorderColor = Gray,
                            unfocusedBorderColor = Black,
                            disabledTextColor = Black,
                            disabledBorderColor = Gray,
                            disabledLabelColor = Gray
                        )
                    )
                        Text(
                        text = if(ativo) "Ativo" else "Inativo",
                        fontWeight = FontWeight.Bold,
                        modifier = Modifier
                            .padding(top = 16.dp),
                            color = if(ativo) FechaduraColor else Color(0xFFFF6B6B)
                    )
                    Switch(
                        checked = ativo,
                        onCheckedChange = { ativo = it },
                        colors = SwitchDefaults.colors(
                            checkedTrackColor = Color(0xFF9FD19C),
                            checkedThumbColor = FechaduraColor,
                            uncheckedThumbColor = Color(0xFFFF6B6B),
                            uncheckedTrackColor = Color(0xFFFFCDD2),
                            disabledCheckedThumbColor = Color.Gray,
                            disabledUncheckedThumbColor = Color.Gray,
                            disabledCheckedTrackColor = Color.Gray,
                            disabledUncheckedTrackColor = Color.Gray
                        ),
                        thumbContent = if (ativo) {
                            {
                                Icon(
                                    imageVector = Icons.Filled.Check,
                                    contentDescription = null,
                                    modifier = Modifier.size(SwitchDefaults.IconSize),
                                )
                            }
                        } else {
                            null
                        }
                    )
                }

                Button(
                    onClick = {
                        navController.popBackStack()
                    },
                    Modifier
                        .fillMaxWidth()
                        .height(55.dp),
                    colors = ButtonDefaults.buttonColors(
                        containerColor = FechaduraColor
                    )
                ) {
                    Text(
                        text = "Salvar",
                        color = Color.White
                    )
                }
            }
        }

        if (showBottomSheet) {
            ModalBottomSheet(
                containerColor = Color.White,
                onDismissRequest = { showBottomSheet = false },
                sheetState = sheetState
            ) {
                Column(
                    modifier = Modifier
                        .fillMaxSize()
                        .padding(16.dp)
                ) {
                    Text(
                        text = "Portas",
                        fontSize = 16.sp,
                        style = MaterialTheme.typography.titleLarge,
                        modifier = Modifier
                            .fillMaxWidth()
                            .wrapContentSize(Alignment.Center),
                    )

                    OutlinedTextField(
                        value = searchQuery,
                        onValueChange = { searchQuery = it },
                        label = { Text(text = "Pesquisar portas") },
                        modifier = Modifier.fillMaxWidth()
                    )

                    Spacer(modifier = Modifier.height(16.dp))

                    if (selectedPortas.isNotEmpty()) {
                        LazyRow(
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(vertical = 8.dp),
                            horizontalArrangement = Arrangement.spacedBy(8.dp),
                            contentPadding = PaddingValues(horizontal = 8.dp)
                        ) {
                            items(selectedPortas.toList()) { porta ->
                                Surface(
                                    modifier = Modifier.padding(4.dp),
                                    color = Color(0xFF4CAF50).copy(alpha = 0.1f),
                                    shape = MaterialTheme.shapes.small,
                                    onClick = {
                                        val updatedSelection = selectedPortas - porta
                                        selectedPortas = updatedSelection
                                        selectedText = if (selectedPortas.isEmpty()) {
                                            "Selecione os grupos de portas"
                                        } else {
                                            selectedPortas.joinToString(", ")
                                        }
                                    }
                                ) {
                                    Row(
                                        verticalAlignment = Alignment.CenterVertically,
                                        modifier = Modifier.padding(8.dp)
                                    ) {
                                        Text(
                                            text = porta,
                                            color = Color(0xFF4CAF50),
                                            style = MaterialTheme.typography.bodyMedium
                                        )
                                        Spacer(modifier = Modifier.width(4.dp))
                                        IconButton(onClick = {
                                            val updatedSelection = selectedPortas - porta
                                            selectedPortas = updatedSelection
                                            selectedText = if (selectedPortas.isEmpty()) {
                                                "Selecione os grupos de portas"
                                            } else {
                                                selectedPortas.joinToString(", ")
                                            }
                                        }) {
                                            Icon(
                                                imageVector = Icons.Default.Close,
                                                contentDescription = "Remover porta",
                                                tint = Color(0xFF4CAF50)
                                            )
                                        }
                                    }
                                }
                            }
                        }
                    }

                    Spacer(modifier = Modifier.height(8.dp))

                    val filteredPortas = portas
                        .filter { it.descricao.contains(searchQuery, ignoreCase = true) }
                        .filterNot { it.descricao in selectedPortas }

                    LazyColumn(
                        verticalArrangement = Arrangement.spacedBy(8.dp),
                        modifier = Modifier.fillMaxSize()
                    ) {
                        items(filteredPortas) { portas ->
                            Column(
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .clickable {
                                        val updatedSelection =
                                            if (selectedPortas.contains(portas.descricao)) {
                                                selectedPortas - portas.descricao
                                            } else {
                                                selectedPortas + portas.descricao
                                            }
                                        selectedPortas = updatedSelection
                                        selectedText = if (selectedPortas.isEmpty()) {
                                            "Selecione os grupos de portas"
                                        } else {
                                            selectedPortas.joinToString(", ")
                                        }
                                    }
                                    .padding(vertical = 8.dp)
                                    .background(
                                        if (selectedPortas.contains(portas.descricao))
                                            Color(0x1F4CAF50)
                                        else Color.Transparent
                                    )
                            ) {
                                Text(
                                    text = portas.descricao,
                                    modifier = Modifier
                                        .fillMaxWidth()
                                        .padding(16.dp),
                                    style = MaterialTheme.typography.bodyLarge
                                )
                                Divider()
                            }
                        }
                    }

                    Spacer(modifier = Modifier.height(16.dp))

                    Button(onClick = {
                        scope.launch { sheetState.hide() }.invokeOnCompletion {
                            if (!sheetState.isVisible) {
                                showBottomSheet = false
                            }
                        }
                    }) {
                        Text(text = "Confirmar")
                    }
                }
            }
        }
    }
}

