import android.widget.Toast
import androidx.compose.foundation.background
import androidx.compose.foundation.clickable
import androidx.compose.foundation.interaction.MutableInteractionSource
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.PaddingValues
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.layout.width
import androidx.compose.foundation.layout.wrapContentSize
import androidx.compose.foundation.lazy.LazyColumn
import androidx.compose.foundation.lazy.LazyRow
import androidx.compose.foundation.lazy.items
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.automirrored.filled.ArrowBack
import androidx.compose.material.icons.filled.ArrowDropDown
import androidx.compose.material.icons.filled.Check
import androidx.compose.material.icons.filled.Close
import androidx.compose.material.icons.filled.Edit
import androidx.compose.material.icons.filled.MoreVert
import androidx.compose.material.icons.filled.Person
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Checkbox
import androidx.compose.material3.Divider
import androidx.compose.material3.DropdownMenu
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.ExposedDropdownMenuDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.IconButton
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.MaterialTheme.colorScheme
import androidx.compose.material3.ModalBottomSheet
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Scaffold
import androidx.compose.material3.Surface
import androidx.compose.material3.Switch
import androidx.compose.material3.SwitchDefaults
import androidx.compose.material3.Text
import androidx.compose.material3.TextField
import androidx.compose.material3.TopAppBar
import androidx.compose.material3.TopAppBarDefaults
import androidx.compose.material3.rememberModalBottomSheetState
import androidx.compose.runtime.Composable
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.rememberCoroutineScope
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.platform.LocalContext
import androidx.compose.ui.semantics.Role
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.tooling.preview.Preview
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import androidx.navigation.compose.rememberNavController
import com.br.fechadura.mobile.sampledata.sampleData
import com.br.fechadura.mobile.sampledata.sampleDataDoors

import com.br.fechadura.mobile.sampledata.sampleDataGroupDoors
import com.br.fechadura.mobile.sampledata.sampleDataGroupUser
import com.br.fechadura.mobile.ui.theme.FechaduraColor
import com.br.fechadura.mobile.ui.theme.onPrimaryLight
import com.br.fechadura.mobile.ui.theme.outlineLight
import com.br.fechadura.mobile.ui.theme.primaryContainerLight
import com.br.fechadura.mobile.ui.theme.primaryLight
import com.br.fechadura.mobile.ui.theme.secondaryLight

import kotlinx.coroutines.launch

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun CadastrarGrupoUsuario(navController: NavController) {

    var nome by remember { mutableStateOf("") }
    var descricao by remember { mutableStateOf("") }
    var ativo by remember { mutableStateOf(true) }

    var selectedUsuarios by remember { mutableStateOf(setOf<String>()) }
    var selectedTextUsuarios by remember { mutableStateOf("Selecione os usuários") }
    var searchQueryUsuarios by remember { mutableStateOf("") }

    var selectedPortas by remember { mutableStateOf(setOf<String>()) }
    var selectedTextPortas by remember { mutableStateOf("Selecione as portas") }
    var searchQueryPortas by remember { mutableStateOf("") }

    val sampleDataUsuarios = sampleData // Lista completa de usuários
    val sampleDataPortas = sampleDataDoors // Lista completa de portas

    val sheetState = rememberModalBottomSheetState()
    val scope = rememberCoroutineScope()
    var showBottomSheetPortas by remember { mutableStateOf(false) }
    var showBottomSheetUsuarios by remember { mutableStateOf(false) }

    Scaffold(
        topBar = {
            TopAppBar(
                title = { Text("Cadastrar Grupo", color = Color.White) },
                colors = TopAppBarDefaults.topAppBarColors(
                    containerColor = FechaduraColor
                ),
                navigationIcon = {
                    IconButton(onClick = { navController.popBackStack() }) {
                        Icon(
                            imageVector = Icons.AutoMirrored.Filled.ArrowBack,
                            contentDescription = "Voltar para início",
                            tint = Color.White
                        )
                    }
                }
            )
        }
    ) { innerPadding ->
        Box(
            modifier = Modifier
                .fillMaxSize()
                .background(Color.White)
                .padding(innerPadding)
        ) {

            Column(
                modifier = Modifier
                    .fillMaxSize()
                    .padding(20.dp),
                verticalArrangement = Arrangement.SpaceBetween
            ) {
                Column(
                    horizontalAlignment = Alignment.CenterHorizontally,
                    verticalArrangement = Arrangement.spacedBy(
                        14.dp,
                        Alignment.CenterVertically
                    )
                ) {

                    OutlinedTextField(
                        value = nome,
                        onValueChange = { nome = it },
                        label = {
                            Text(
                                "Nome",
                                color = FechaduraColor,
                                fontSize = 13.sp
                            )
                        },
                        modifier = Modifier
                            .fillMaxWidth(),
                        textStyle = TextStyle(color = Color.Black),
                        singleLine = true
                    )

                    OutlinedTextField(
                        value = descricao,
                        onValueChange = { descricao = it },
                        label = {
                            Text(
                                "Descrição",
                                color = FechaduraColor,
                                fontSize = 13.sp
                            )
                        },
                        modifier = Modifier
                            .fillMaxWidth(),
                        textStyle = TextStyle(color = Color.Black),
                        singleLine = true
                    )

                    OutlinedTextField(
                        value = selectedTextUsuarios,
                        onValueChange = { },
                        label = {
                            Text(
                                "Usuários",
                                color = FechaduraColor
                            )
                        },
                        modifier = Modifier
                            .fillMaxWidth(),
                        enabled = false,
                        singleLine = true,
                        textStyle = TextStyle(color = Color.Black),
                        trailingIcon = {
                            IconButton(onClick = { showBottomSheetUsuarios = true }) {
                                Icon(
                                    imageVector = Icons.Default.ArrowDropDown,
                                    contentDescription = "Abrir menu",
                                    tint = Color.Gray
                                )
                            }
                        },
                    )

                    OutlinedTextField(
                        value = selectedTextPortas,
                        onValueChange = { },
                        label = {
                            Text(
                                "Portas",
                                color = FechaduraColor
                            )
                        },
                        modifier = Modifier
                            .fillMaxWidth(),
                        enabled = false,
                        singleLine = true,
                        textStyle = TextStyle(color = Color.Black),
                        trailingIcon = {
                            IconButton(onClick = { showBottomSheetPortas = true }) {
                                Icon(
                                    imageVector = Icons.Default.ArrowDropDown,
                                    contentDescription = "Abrir menu",
                                    tint = Color.Gray
                                )
                            }
                        },
                    )

                    val interactionSource = remember {
                        MutableInteractionSource()
                    }
                    Row(
                        modifier = Modifier
                            .clickable(
                                interactionSource = interactionSource,
                                indication = null,
                                role = Role.Switch,
                                onClick = {
                                    ativo = !ativo
                                }
                            )
                            .fillMaxWidth()
                            .padding(top = 10.dp),
                        verticalAlignment = Alignment.CenterVertically
                    ) {
                        Text(
                            text = "Ativo",
                            fontWeight = FontWeight.Bold,
                            modifier = Modifier
                                .padding(end = 10.dp),
                            color = outlineLight
                        )

                        Switch(
                            checked = ativo,
                            onCheckedChange = { ativo = it },
                            thumbContent = if (ativo) {
                                {
                                    Icon(
                                        imageVector = Icons.Filled.Check,
                                        contentDescription = null,
                                        modifier = Modifier.size(SwitchDefaults.IconSize),
                                    )
                                }
                            } else {
                                null
                            },
                            colors = SwitchDefaults.colors(
                                checkedThumbColor = primaryLight,
                                checkedTrackColor = primaryContainerLight,
                                uncheckedThumbColor = onPrimaryLight,
                                uncheckedTrackColor = secondaryLight
                            )
                        )
                    }
                }
                Button(
                    onClick = { /* TODO */ },
                    Modifier
                        .fillMaxWidth()
                        .height(55.dp),
                    colors = ButtonDefaults.buttonColors(
                        containerColor = FechaduraColor
                    )
                ) {
                    Text(
                        text = "Cadastrar",
                        color = Color.White
                    )
                }
            }
        }
        // Portas
        if (showBottomSheetPortas) {
            ModalBottomSheet(
                containerColor = Color.White,
                onDismissRequest = { showBottomSheetPortas = false },
                sheetState = sheetState
            ) {
                Column(
                    modifier = Modifier
                        .fillMaxSize()
                        .padding(16.dp)
                ) {
                    Text(
                        text = "Portas",
                        fontSize = 16.sp,
                        style = MaterialTheme.typography.titleLarge,
                        modifier = Modifier
                            .fillMaxWidth()
                            .wrapContentSize(Alignment.Center),
                    )

                    OutlinedTextField(
                        value = searchQueryPortas,
                        onValueChange = { searchQueryPortas = it },
                        label = { Text(text = "Pesquisar portas") },
                        modifier = Modifier.fillMaxWidth()
                    )

                    Spacer(modifier = Modifier.height(16.dp))

                    if (selectedPortas.isNotEmpty()) {
                        LazyRow(
                            modifier = Modifier
                                .fillMaxWidth()
                                .padding(vertical = 8.dp),
                            horizontalArrangement = Arrangement.spacedBy(8.dp),
                            contentPadding = PaddingValues(horizontal = 8.dp)
                        ) {
                            items(selectedPortas.toList()) { porta ->
                                Surface(
                                    modifier = Modifier.padding(4.dp),
                                    color = Color(0xFF4CAF50).copy(alpha = 0.1f),
                                    shape = MaterialTheme.shapes.small,
                                    onClick = {
                                        val updatedSelection = selectedPortas - porta
                                        selectedPortas = updatedSelection
                                        selectedTextPortas =
                                            if (selectedPortas.isEmpty()) {
                                                "Selecione as portas"
                                            } else {
                                                selectedPortas.joinToString(", ")
                                            }
                                    }
                                ) {
                                    Row(
                                        verticalAlignment = Alignment.CenterVertically,
                                        modifier = Modifier.padding(8.dp)
                                    ) {
                                        Text(
                                            text = porta,
                                            color = Color(0xFF4CAF50),
                                            style = MaterialTheme.typography.bodyMedium
                                        )
                                        Spacer(modifier = Modifier.width(4.dp))
                                        IconButton(onClick = {
                                            val updatedSelection =
                                                selectedPortas - porta
                                            selectedPortas = updatedSelection
                                            selectedTextPortas =
                                                if (selectedPortas.isEmpty()) {
                                                    "Selecione as portas"
                                                } else {
                                                    selectedPortas.joinToString(", ")
                                                }
                                        }) {
                                            Icon(
                                                imageVector = Icons.Default.Close,
                                                contentDescription = "Remover porta",
                                                tint = Color(0xFF4CAF50)
                                            )
                                        }
                                    }
                                }
                            }
                        }
                    }

                    Spacer(modifier = Modifier.height(8.dp))

                    val filteredPortas = sampleDataPortas
                        .filter {
                            it.descricao.contains(
                                searchQueryPortas,
                                ignoreCase = true
                            )
                        }
                        .filterNot { it.descricao in selectedPortas }

                    LazyColumn(
                        verticalArrangement = Arrangement.spacedBy(8.dp),
                        modifier = Modifier.fillMaxSize()
                    ) {
                        items(filteredPortas) { porta ->
                            Column(
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .clickable {
                                        val updatedSelection =
                                            if (selectedPortas.contains(porta.descricao)) {
                                                selectedPortas - porta.descricao
                                            } else {
                                                selectedPortas + porta.descricao
                                            }
                                        selectedPortas = updatedSelection
                                        selectedTextPortas =
                                            if (selectedPortas.isEmpty()) {
                                                "Selecione as portas"
                                            } else {
                                                selectedPortas.joinToString(", ")
                                            }
                                    }
                                    .padding(vertical = 8.dp)
                                    .background(
                                        if (selectedPortas.contains(porta.descricao))
                                            Color(0x1F4CAF50)
                                        else Color.Transparent
                                    )
                            ) {
                                Text(
                                    text = porta.descricao,
                                    modifier = Modifier
                                        .fillMaxWidth()
                                        .padding(16.dp),
                                    style = MaterialTheme.typography.bodyLarge
                                )
                            }
                        }
                    }

                    Spacer(modifier = Modifier.height(16.dp))

                    Button(onClick = {
                        scope.launch { sheetState.hide() }.invokeOnCompletion {
                            if (!sheetState.isVisible) {
                                showBottomSheetPortas = false
                            }
                        }
                    }) {
                        Text(text = "Confirmar")
                    }
                }
            }
        }
    }

    // Usuários
    if (showBottomSheetUsuarios) {
        ModalBottomSheet(
            containerColor = Color.White,
            onDismissRequest = { showBottomSheetUsuarios = false },
            sheetState = sheetState
        ) {
            Column(
                modifier = Modifier
                    .fillMaxSize()
                    .padding(16.dp)
            ) {
                Text(
                    text = "Usuários",
                    fontSize = 16.sp,
                    style = MaterialTheme.typography.titleLarge,
                    modifier = Modifier
                        .fillMaxWidth()
                        .wrapContentSize(Alignment.Center),
                )

                OutlinedTextField(
                    value = searchQueryUsuarios,
                    onValueChange = { searchQueryUsuarios = it },
                    label = { Text(text = "Pesquisar usuário") },
                    modifier = Modifier.fillMaxWidth(),
                    singleLine = true
                )

                Spacer(modifier = Modifier.height(16.dp))

                if (selectedUsuarios.isNotEmpty()) {
                    LazyRow(
                        modifier = Modifier
                            .fillMaxWidth()
                            .padding(vertical = 8.dp),
                        horizontalArrangement = Arrangement.spacedBy(8.dp),
                        contentPadding = PaddingValues(horizontal = 8.dp)
                    ) {
                        items(selectedUsuarios.toList()) { grupo ->
                            Surface(
                                modifier = Modifier.padding(4.dp),
                                color = Color(0xFF4CAF50).copy(alpha = 0.1f),
                                shape = MaterialTheme.shapes.small,
                                onClick = {
                                    val updatedSelection = selectedUsuarios - grupo
                                    selectedUsuarios = updatedSelection
                                    selectedTextUsuarios = if (selectedUsuarios.isEmpty()) {
                                        "Selecione usuário"
                                    } else {
                                        selectedUsuarios.joinToString(", ")
                                    }
                                }
                            ) {
                                Row(
                                    verticalAlignment = Alignment.CenterVertically,
                                    modifier = Modifier.padding(8.dp)
                                ) {
                                    Text(
                                        text = grupo,
                                        color = Color(0xFF4CAF50),
                                        style = MaterialTheme.typography.bodyMedium
                                    )
                                    Spacer(modifier = Modifier.width(4.dp))
                                    IconButton(onClick = {
                                        val updatedSelection = selectedUsuarios - grupo
                                        selectedUsuarios = updatedSelection
                                        selectedTextUsuarios =
                                            if (selectedUsuarios.isEmpty()) {
                                                "Selecione o usuário"
                                            } else {
                                                selectedPortas.joinToString(", ")
                                            }
                                    }) {
                                        Icon(
                                            imageVector = Icons.Default.Close,
                                            contentDescription = "Remover grupo",
                                            tint = Color(0xFF4CAF50)
                                        )
                                    }
                                }
                            }
                        }
                    }
                }

                Spacer(modifier = Modifier.height(8.dp))

                val filteredUsuarios = sampleDataUsuarios
                    .filter { it.nome.contains(searchQueryUsuarios, ignoreCase = true) }
                    .filterNot { it.nome in selectedUsuarios }

                LazyColumn(
                    verticalArrangement = Arrangement.spacedBy(8.dp),
                    modifier = Modifier.fillMaxSize()
                ) {
                    items(filteredUsuarios) { usuario ->
                        Column(
                            modifier = Modifier
                                .fillMaxWidth()
                                .clickable {
                                    val updatedSelection =
                                        if (selectedUsuarios.contains(usuario.nome)) {
                                            selectedUsuarios - usuario.nome
                                        } else {
                                            selectedUsuarios + usuario.nome
                                        }
                                    selectedUsuarios = updatedSelection
                                    selectedTextUsuarios = if (selectedUsuarios.isEmpty()) {
                                        "Selecione os grupos"
                                    } else {
                                        selectedUsuarios.joinToString(", ")
                                    }
                                }
                                .padding(vertical = 8.dp)
                                .background(
                                    if (selectedUsuarios.contains(usuario.nome))
                                        Color(0x1F4CAF50)
                                    else Color.Transparent
                                )
                        ) {
                            Text(
                                text = usuario.nome,
                                modifier = Modifier
                                    .fillMaxWidth()
                                    .padding(16.dp),
                                style = MaterialTheme.typography.bodyLarge
                            )
                        }
                    }
                }

                Spacer(modifier = Modifier.height(16.dp))

                Button(onClick = {
                    scope.launch { sheetState.hide() }.invokeOnCompletion {
                        if (!sheetState.isVisible) {
                            showBottomSheetUsuarios = false
                        }
                    }
                }) {
                    Text(text = "Confirmar")
                }
            }
        }
    }
}


@Preview(showSystemUi = true)
@Composable
private fun CadastrarGrupoUsuarioPreview() {
    CadastrarGrupoUsuario(navController = rememberNavController())

}
